/* Name: main.c
 * Project: custom-class, a basic USB example
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-09
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 790 2010-05-30 21:00:26Z cs $
 */

/*
This example should run on most AVRs with only little changes. No special
hardware resources except INT0 are used. You may have to change usbconfig.h for
different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
at least be connected to INT0 as well.
We assume that an LED is connected to port B bit 0. If you connect it to a
different port or bit, change the macros below:
*/
#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <avr/wdt.h>
#include <util/delay.h>
#include <atm88lib.h>

#define ENC_CTL	DDRD	//encoder port control
#define ENC_WR	PORTD	//encoder port write	
#define ENC_RD	PIND	//encoder port read
#define ENC_A 2	
#define ENC_B 0	

#define LED_STATE_ON 0
#define LED_STATE_OFF 1
#define LED_STATE_COLOR 2

unsigned char color = 255;
unsigned char brightness = 0;
unsigned char ledState = LED_STATE_ON;

void setColor(unsigned char);
void turnOff();
void turnOn();
void setUpADC();
void readADC();

int __attribute__((noreturn)) main(void)
{
    wdt_disable();

    DDRB |= _BV(PB3);
    DDRD |= _BV(PD5);
    DDRD |= _BV(PD6);

    PORTB &= ~_BV(PB3);
    PORTD &= ~_BV(PD5);
    PORTD &= ~_BV(PD6);
    
    DDRD &= ~_BV(PD2);
    PORTD |= _BV(PD2); //PD2/PCINT18 input, pullup active

    DDRD &= ~_BV(PD0);
    PORTD |= _BV(PD0); //PD0/PCINT16 input, pullup active

    DDRD |= _BV(PD1);
    PORTD &= ~_BV(PD1); //PD1 putput, low
    
    PCMSK2 |= _BV(PCINT16) | _BV(PCINT18);
    PCICR |= _BV(PCIE2);
    
    DDRD |= _BV(PD4);//ground for button
    DDRD &= ~_BV(PD3);//button input
    PORTD |= _BV(PD3);//button pullup;
    
    EICRA |= _BV(ISC11);//
    EICRA &= ~_BV(ISC10);//int1 detection on falling edge
    EIMSK |= _BV(INT1);//enable int1 detection
    
    DDRC |= _BV(PC4) | _BV(PC2);
    PORTC |= _BV(PC4);
    PORTC &= ~_BV(PC2);
    DDRC &= ~_BV(PC1);
    PORTC &= ~_BV(PC1);
    setUpADC();
    
    StartPWM0(PRESC1, PWMA_CLEAR, PWMB_CLEAR);
    StartPWM2(PRESC1, PWMA_CLEAR, PWMB_NONE);
    
    sei();
    _delay_ms(10);//tik paleidus ivyksta enkoderio interruptas, todėl palaukiam
    turnOn();
//    ADCSRA |= _BV(ADSC);//start first conversion
    for(;;){
        readADC();
        _delay_ms(1);
    }
}

void setColor(unsigned char color) {
    if(color <= 84) {
        OCR0B = (84 - color) * brightness / 84;
        OCR0A = color * brightness / 84;
        OCR2A = 0;
    } else if(color > 84 && color <= 168) {
        OCR0B = 0;
        OCR0A = (168 - color) * brightness / 84;
        OCR2A = (color - 84) * brightness / 84;
    } else if(color > 168 && color <= 252) {
        OCR0B = (color - 168) * brightness / 84;
        OCR0A = 0;
        OCR2A = (252 - color) * brightness / 84;
    }
}

void turnOff() {
    ledState = LED_STATE_OFF;
    OCR0B = 0;
    OCR0A = 0;
    OCR2A = 0;  
}

void turnOn() {
    ledState = LED_STATE_ON;
    OCR0B = brightness;
    OCR0A = brightness;
    OCR2A = brightness;   
}

#define STATE_ON_RIGHT 42
#define STATE_ON_BOTH 43
#define STATE_OFF_BOTH 44

uint8_t lastState = STATE_OFF_BOTH;

ISR(PCINT2_vect) {
    uint8_t state = STATE_OFF_BOTH;
    
    uint8_t a = !(PIND & _BV(PD2));
    uint8_t b = !(PIND & _BV(PD0));
    
    if(a && !b){
        state = STATE_ON_RIGHT;
    } else if(a && b){
        state = STATE_ON_BOTH;
    } else if(!a && b){
        //DOES NOT HAPPEN
    } else if(!a && !b) {
        state = STATE_OFF_BOTH;
    }
    
    if( ((state == STATE_ON_BOTH && lastState == STATE_OFF_BOTH) || state == STATE_ON_RIGHT && lastState == STATE_ON_BOTH)) {
        color++;
    } else if((state == STATE_ON_RIGHT && lastState == STATE_OFF_BOTH) || (state == STATE_ON_BOTH && lastState == STATE_ON_RIGHT)) {
        color--;
    }
    lastState = state;
    ledState = LED_STATE_COLOR;
    setColor(color);
}

int isOn() {
    return LED_STATE_ON == ledState;
}

int isOff() {
    return LED_STATE_OFF == ledState;
}

int isColor() {
    return LED_STATE_COLOR == ledState;
}


ISR(INT1_vect) {
        if(isOn()){
            turnOff();
        } else {
            turnOn();
        }
}

void setUpADC() {
    ADMUX |= _BV(REFS0);//AVcc with external capacitor at AREF pin
    ADMUX |= _BV(MUX1) | _BV(MUX0);//ADC3 as input
    ADMUX &= ~_BV(ADLAR);
    ADCSRA |= _BV(ADEN) ;//| _BV(ADIE)| _BV(ADATE)  | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0)
}

void readADC() {
	ADCSRA |= _BV(ADSC);
	while(ADCSRA & _BV(ADSC));
    uint16_t adcResult = (ADCL) | ((ADCH) << 8);

    brightness = ((float)adcResult / (float) 1024) * (float)255;
    if(isOn()) {
        turnOn();
    } else if(isOff()) {
        //NOTHING
    } else if(isColor()){
        setColor(color);
    }
}

ISR(ADC_vect) {
}


